class ChunksController < ApplicationController
  layout nil

  before_action :set_learning_object

  #GET /chunk
  def show
    chunk = resumable_chunk chunk_number

    unless valid_mime_type?
      render :nothing => true, :status => 415
    else
      if File.exists?(chunk)
        post_file_and_create_thumbnail @learning_object, resumable_filename if last_chunk?
        #Let resumable.js know this chunk already exists
        render :nothing => true, :status => 200
      else
        #Let resumable.js know this chunk doesnt exists and needs to be uploaded
        render :nothing => true, :status => 404
      end
    end
  end
  #POST /chunk
  def create

    unless valid_mime_type?
      return render :nothing => true, :status => 415
    end

    #chunk path based on the parameters
    chunk = resumable_chunk chunk_number

    #Create chunks directory when not present on system
    if !File.directory?(dir)
      FileUtils.mkdir(dir, :mode => 0700)
    end

    #Move the uploaded chunk to the directory
    FileUtils.mv chunk_tmpfile, chunk

    #Concatenate all the partial files into the original file
    #When all chunks are uploaded
    if last_chunk?
      #Create a target file
      File.open(resumable_filename, "a") do |target|
        #Loop trough the chunks
        for i in 1..chunk_number
          #Select the chunk
          chunk = File.open(resumable_chunk(i), 'r').read

          #Write chunk into target file
          chunk.each_line do |line|
            target << line
          end

          #Deleting chunk
          FileUtils.rm resumable_chunk(i), :force => true
        end
        puts "File saved to #{resumable_filename}"
      end

      post_file_and_create_thumbnail @learning_object, resumable_filename
    end

    render nothing: true, status: 200
  end

  private

  def post_file_and_create_thumbnail(learning_object, filename)
    publisher = LearningObjectPublisher.new(DspaceService.create_client)
    publisher.post learning_object, filename
  end

  def chunk_tmpfile
    chunks_params[:file].tempfile
  end

  def last_chunk?
    chunk_number == total_chunks
  end

  def chunk_size
    chunks_params[:resumableChunkSize].to_i
  end

  def total_chunks
    chunks_params[:resumableTotalChunks].to_i
  end

  def chunk_number
    chunks_params[:resumableChunkNumber].to_i
  end

  def resumable_chunk(part)
    "#{resumable_filename}.part#{part}"
  end

  def resumable_filename
    "#{dir}/#{chunks_params[:resumableFilename]}"
  end

  def dir
    "/tmp/#{chunks_params[:resumableIdentifier]}"
  end

  def set_learning_object
    @learning_object = LearningObject.find chunks_params[:learning_object_id]
  end

  def resumable_file_extension
    File.extname(chunks_params[:resumableFilename])[1..-1]
  end

  def valid_mime_type?
    @learning_object.object_type.mime_types.map(&:extension).include? resumable_file_extension
  end
  # Never trust parameters from the scary internet, only allow the white list through.
  def chunks_params
    params.permit(:file, :learning_object_id, :resumableIdentifier, :resumableFilename, :resumableChunkNumber, :resumableTotalChunks, :resumableChunkSize)
  end
end
