namespace :language do
  desc 'Clean languages that fail the validations'
  task :clean => :environment do
    valid_lang = Language.find(1)
    Language.all.each do |language|
      if language.invalid?
        language.learning_objects.each do |lo|
          lo.update_attribute(:language, valid_lang)
        end
        language.update(learning_objects: [])
        Language.destroy language.id
      end
    end
  end

  desc 'Destroy languages that are not defaults'
  task :restart => :environment do
    default_language = Language.find_or_create_by(name:'Outro') do |l|
      l.code = 'Outro'
    end
    Language.find_each do |language|
      unless is_valid? language
        LearningObject.where(language: language).update_all(language_id: default_language.id)
        language.destroy
      end
    end
  end

  def is_valid?(language)
    if default_languages.has_key? language.name and default_languages[language.name] == language.code
      return true
    else
      return false
    end
  end

  def default_languages
    {
        'Português' => 'pt',
        'Inglês' => 'en',
        'Espanhol' => 'es',
        'Francês' => 'fr',
        'Alemão' => 'de',
        'Italiano'=> 'it',
        'Mandarim' => 'zh',
        'Russo' => 'ru',
        'Japonês' => 'jp',
        'Outro' => 'Outro'
    }
  end
end